/* getdir.c -- Functions to read a directory into *files.
 *
 * Last revised: 11 June 1996
 * Written (sort of) Bryan Herger
 *
 * See COPYING for copyright and license information.
 */

/* Standard includes */
#include <Xm/Xm.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "filearea.h"

extern Widget top_level, fm_textfield;

/* This struct holds file data */
struct filedata {
  char *name;
  char *linkto;
  int filetype;
};

/* This holds the list of files in the current directory */
struct filedata *files;

/* The name of the currently open directory */
char current_wd[120];

/* This is the number of file entries allocated in *files */
/* fileptr is first unused space */
int nfiles = 100;
int fileptr;

/* A few functions */
int load_dir(const char *);
void clear_dir_list();
int compare_name();
void sort_dir();

char *getparent() {
  int x;
  char *to_return;

  for (x = strlen(current_wd); x > 0; x--)
    if (current_wd[x] == '/')
      break;

  if (x == 0) x++;
  to_return = malloc(sizeof(char) * (x+1));
  strncpy(to_return, current_wd, x);
  to_return[x] = '\0';
  if (x == 0)
    sprintf(to_return, "/");
#ifdef DEBUG
  printf("%d:%s\n", x, to_return);
#endif
  return(to_return);
}
  
void clear_dir_list ()
{
  register int i;

  for (i = 0; i < fileptr; i++)
    {
      free (files[i].name);
      /* if (files[i].filetype == 2)
        free (files[i].linkto); */
    }

  fileptr = 0;
}

int load_dir (const char *name)
{
  register DIR *reading;
  register struct dirent *next;
  char fullpath[120], newtitle[128];
  int linkadjust = 0, x;
  struct stat buf;

  errno = 0;
  reading = opendir (name);
  if (!reading)
    {
      return 1;
    }

  clear_dir_list();
  /*  if (files != NULL) {
    nfiles = 100;
    files = realloc(files, sizeof(struct filedata) * nfiles);
  } */

  while ((next = readdir (reading)) != NULL) {
    files[fileptr].name = malloc(sizeof(char) * (strlen(next->d_name)+1));
    strcpy(files[fileptr].name, next->d_name);
    sprintf(fullpath, "%s/%s", name, next->d_name);
    lstat(fullpath, &buf);
    if (S_ISLNK(buf.st_mode)) {
      linkadjust = 1;
    }
    if (S_ISDIR(buf.st_mode))
      files[fileptr].filetype = 0;
    else
      files[fileptr].filetype = 2;
    files[fileptr].filetype += linkadjust;
    fileptr++;
    if ((fileptr + 10) > nfiles) {
      nfiles += 100;
      files = realloc(files, sizeof(struct filedata) * nfiles);
    }
  }

  sort_dir();

  sprintf(current_wd, "%s", name);
  sprintf(newtitle, "bxfm: %s", current_wd);

  XtVaSetValues(top_level, XmNtitle, newtitle, NULL);
  XtVaSetValues(fm_textfield, XmNvalue, current_wd, NULL);

  return 0;
}

void sort_dir()
{
  int (*func)();

  func = compare_name;

  qsort(files, fileptr, sizeof (struct filedata), func);
}

int compare_name (file1, file2)
     struct filedata *file1, *file2;
{
  return strcmp (file1->name, file2->name);
}

int refresh() {
  load_dir(current_wd);
  DrawFileArea();
}












